package binarytree.core;

public class TreeNode<T extends Comparable<T>> {
	private TreeNode<T> left, right;
	private T value;
	private int level;
	
	public TreeNode(T value) {
		this.value = value;
		this.level = 0;
	}
	
	void set(T value) {
		this.value = value;
	}
	
	void remove(Direction which) {
		if (which == Direction.LEFT) {
			this.left = null;
		} else if (which == Direction.RIGHT) {
			this.right = null;
		}
	}
	
	void setChild(Direction which, TreeNode<T> node) {
		node.setLevel(this.level + 1);
		if (which == Direction.LEFT) {
			this.left = node;
		} else if (which == Direction.RIGHT) {
			this.right = node;
		}
	}
	
	void setLevel(int newLevel) {
		this.level = newLevel;
	}
	
	public int getLevel() {return level;}
	
	public int size() {return 1 + (hasLeft() ? left.size() : 0) + (hasRight() ? right.size() : 0);}
	
	public T get() {return value;}
	
	public T getMin() {return hasLeft() ? left.getMin() : get();}
	public T getMax() {return hasRight() ? right.getMax() : get();}
	
	public boolean hasLeft() {return left != null;}
	public TreeNode<T> getLeft() {return left;}
	
	public boolean hasRight() {return right != null;}
	public TreeNode<T> getRight() {return right;}
	
	public boolean isLeaf() {return left == null && right == null;}
}
